@@ -1,23 +1,23 @@
 """
 Markov based methods for spatial dynamics
 """
-__author__ =  "Sergio J. Rey <srey@asu.edu"
+__author__ = "Sergio J. Rey <srey@asu.edu"
 
 import numpy as np
 import numpy.linalg as la
-from ergodic import fmpt, steady_state
+from pysal.spatial_dynamics.ergodic import fmpt, steady_state
 from scipy import stats
 import pysal
 from operator import gt
 
 __all__ = ["Markov", "LISA_Markov", "Spatial_Markov", "kullback",
-       "prais", "shorrock"]
+           "prais", "shorrock"]
 
 # TT predefine LISA transitions
 # TT[i,j] is the transition type from i to j
 # i = quadrant in period 0
 # j = quadrant in period 1
-# uses one offset so first row and col of TT are ignored 
+# uses one offset so first row and col of TT are ignored
 TT = np.zeros((5, 5), int)
 c = 1
 for i in range(1, 5):
@@ -35,9 +35,9 @@ for i in range(1, 5):
 MOVE_TYPES = {}
 c = 1
 cases = (True, False)
-sig_keys = [ (i,j) for i in cases for j in cases ]
+sig_keys = [(i, j) for i in cases for j in cases]
 
-for i,sig_key in enumerate(sig_keys):
+for i, sig_key in enumerate(sig_keys):
     c = 1 + i * 16
     for i in range(1, 5):
         for j in range(1, 5):
@@ -45,16 +45,17 @@ for i,sig_key in enumerate(sig_keys):
             MOVE_TYPES[key] = c
             c += 1
 
+
 class Markov:
     """
     Classic Markov transition matrices
 
     Parameters
     ----------
-    class_ids    : array (n, t) 
+    class_ids    : array (n, t)
                    One row per observation, one column recording the state of each
                    observation, with as many columns as time periods
-    classes      : array (k) 
+    classes      : array (k)
                    All different classes (bins) of the matrix
 
     Attributes
@@ -132,9 +133,10 @@ class Markov:
             [ 0.21499942],
             [ 0.21134662],
             [ 0.17776576]])
-    
+
     """
     def __init__(self, class_ids, classes=[]):
+        #pylint; Dangerous default value [] as argument
         if len(classes):
             self.classes = classes
         else:
@@ -142,14 +144,14 @@ class Markov:
 
         n, t = class_ids.shape
         k = len(self.classes)
-        js = range(t-1)
+        js = range(t - 1)
 
         classIds = self.classes.tolist()
-        transitions=np.zeros((k,k))
+        transitions = np.zeros((k, k))
         for state_0 in js:
-            state_1 = state_0+1
-            state_0 = class_ids[:,state_0]
-            state_1 = class_ids[:,state_1]
+            state_1 = state_0 + 1
+            state_0 = class_ids[:, state_0]
+            state_1 = class_ids[:, state_1]
             initial = np.unique(state_0)
             for i in initial:
                 ending = state_1[state_0 == i]
@@ -160,35 +162,27 @@ class Markov:
                     transitions[row, col] += sum(ending == j)
         self.transitions = transitions
         row_sum = transitions.sum(axis=1)
-        p = np.dot(np.diag(1/(row_sum+(row_sum == 0))), transitions)
+        p = np.dot(np.diag(1 / (row_sum + (row_sum == 0))), transitions)
         self.p = np.matrix(p)
 
-        # steady_state vector 
+        # steady_state vector
         v, d = la.eig(np.transpose(self.p))
         # for a regular P maximum eigenvalue will be 1
         mv = max(v)
         # find its position
         i = v.tolist().index(mv)
         # normalize eigenvector corresponding to the eigenvalue 1
-        self.steady_state =  d[:,i]/sum(d[:,i])
-
-
-        
-
-
-
-
-
+        self.steady_state = d[:, i] / sum(d[:, i])
 
 
 class Spatial_Markov:
     """
     Markov transitions conditioned on the value of the spatial lag
-    
+
     Parameters
     ----------
 
-    y               : array (n,t) 
+    y               : array (n,t)
                       One row per observation, one column per state of each
                       observation, with as many columns as time periods
 
@@ -240,7 +234,7 @@ class Spatial_Markov:
                       of the difference between the conditional transition
                       matrix against the overall transition matrix, first
                       element of the tuple is the chi2 value, second its
-                      p-value and the third the degrees of freedom 
+                      p-value and the third the degrees of freedom
     x2              : float
                       sum of the chi2 values for each of the conditional tests
                       has an asymptotic chi2 distribution with k(k-1)(k-1)
@@ -254,9 +248,9 @@ class Spatial_Markov:
                       distribution
     x2_rpvalue       : float (if permutations>0)
                       pseudo p-value for x2 based on random spatial permutations
-                      of the rows of the original transitions 
+                      of the rows of the original transitions
     x2_realizations : array (permutations,1)
-                      the values of x2 for the random permutations 
+                      the values of x2 for the random permutations
 
     Notes
     -----
@@ -267,7 +261,7 @@ class Spatial_Markov:
     preferable since it simulates the randomness under the null. It is an
     experimental test requiring further analysis.
 
-    
+
     Examples
     --------
     >>> import pysal
@@ -280,7 +274,7 @@ class Spatial_Markov:
     >>> sm = Spatial_Markov(rpci, w, fixed=True, k=5)
     >>> for p in sm.P:
     ...     print p
-    ...     
+    ...
     [[ 0.96341463  0.0304878   0.00609756  0.          0.        ]
      [ 0.06040268  0.83221477  0.10738255  0.          0.        ]
      [ 0.          0.14        0.74        0.12        0.        ]
@@ -306,13 +300,13 @@ class Spatial_Markov:
      [ 0.01149425  0.16091954  0.74712644  0.08045977  0.        ]
      [ 0.          0.01036269  0.06217617  0.89637306  0.03108808]
      [ 0.          0.          0.          0.02352941  0.97647059]]
- 
+
     The probability of a poor state remaining poor is 0.963 if their
     neighbors are in the 1st quintile and 0.798 if their neighbors are
     in the 2nd quintile. The probability of a rich economy remaining
     rich is 0.977 if their neighbors are in the 5th quintile, but if their
     neighbors are in the 4th quintile this drops to 0.903.
-   
+
     Test if the transitional dynamics are homogeneous across the lag classes
 
     >>> sm.x2
@@ -336,7 +330,7 @@ class Spatial_Markov:
 
     >>> for f in sm.F:
     ...     print f
-    ...     
+    ...
     [[   2.29835259   28.95614035   46.14285714   80.80952381  279.42857143]
      [  33.86549708    3.79459555   22.57142857   57.23809524  255.85714286]
      [  43.60233918    9.73684211    4.91085714   34.66666667  233.28571429]
@@ -383,19 +377,19 @@ class Spatial_Markov:
             [  6.33129261e+01,   5.83089133e-13,   4.00000000e+00],
             [  7.22778509e+01,   7.54951657e-15,   4.00000000e+00],
             [  2.32659201e+02,   0.00000000e+00,   4.00000000e+00]])
-    
+
 
     References
     ----------
 
     .. [1] Rey, S.J. 2001. "Spatial empirics for economic growth
        and convergence", 34 Geographical Analysis, 33, 195-214.
-    
+
     """
     def __init__(self, y, w, k=4, permutations=0, fixed=False):
 
         self.y = y
-        rows,cols = y.shape
+        rows, cols = y.shape
         self.cols = cols
         npm = np.matrix
         npa = np.array
@@ -406,7 +400,8 @@ class Spatial_Markov:
             yb.shape = (rows, cols)
             classes = yb
         else:
-            classes = npa([pysal.Quantiles(y[:,i],k=k).yb for i in np.arange(cols)]).transpose()
+            classes = npa([pysal.Quantiles(y[:, i], k=k)
+                           .yb for i in np.arange(cols)]).transpose()
         classic = Markov(classes)
         self.classes = classes
         self.p = classic.p
@@ -420,8 +415,8 @@ class Spatial_Markov:
         self.shtest = self._mn_test()
         self.chi2 = self._chi2_test()
         self.x2 = sum([c[0] for c in self.chi2])
-        dof = k * (k-1) * (k-1)
-        self.x2_pvalue = 1 - stats.chi2.cdf(self.x2, dof) 
+        dof = k * (k - 1) * (k - 1)
+        self.x2_pvalue = 1 - stats.chi2.cdf(self.x2, dof)
         self.x2_dof = dof
 
         if permutations:
@@ -432,15 +427,14 @@ class Spatial_Markov:
             x2ss = []
             for perm in range(permutations):
                 T, P, ss, F = self._calc(nrp(y), w, classes, k=k)
-                x2 = [chi2(T[i],self.transitions)[0] for i in range(k)]
+                x2 = [chi2(T[i], self.transitions)[0] for i in range(k)]
                 x2s = sum(x2)
                 x2_realizations[perm] = x2s
                 if x2s >= self.x2:
                     counter += 1
-            self.x2_rpvalue = (counter+1.0)/(permutations+1.)
+            self.x2_rpvalue = (counter + 1.0) / (permutations + 1.)
             self.x2_realizations = x2_realizations
 
-
     def _calc(self, y, w, classes, k):
         # lag markov
         ly = pysal.lag_spatial(w, y)
@@ -450,23 +444,24 @@ class Spatial_Markov:
             l_classes = pysal.Quantiles(ly.flatten(), k=k).yb
             l_classes.shape = ly.shape
         else:
-            l_classes = npa([pysal.Quantiles(ly[:,i],k=k).yb for i in np.arange(self.cols)])
+            l_classes = npa([pysal.Quantiles(
+                ly[:, i], k=k).yb for i in np.arange(self.cols)])
             l_classes = l_classes.transpose()
         l_classic = Markov(l_classes)
         T = np.zeros((k, k, k))
         n, t = y.shape
-        for t1 in range(t-1):
+        for t1 in range(t - 1):
             t2 = t1 + 1
             for i in range(n):
                 T[l_classes[i, t1], classes[i, t1], classes[i, t2]] += 1
 
         P = np.zeros_like(T)
-        F = np.zeros_like(T) # fmpt
+        F = np.zeros_like(T)  # fmpt
         ss = np.zeros_like(T[0])
         for i, mat in enumerate(T):
             row_sum = mat.sum(axis=1)
             row_sum = row_sum + (row_sum == 0)
-            p_i = np.matrix(np.diag(1./row_sum)*np.matrix(mat))
+            p_i = np.matrix(np.diag(1. / row_sum) * np.matrix(mat))
             #print i
             #print mat
             #print p_i
@@ -474,7 +469,8 @@ class Spatial_Markov:
             try:
                 F[i] = fmpt(p_i)
             except:
-                print "Singlular fmpt matrix for class ",i
+                #pylint; "No exception type(s) specified"
+                print "Singlular fmpt matrix for class ", i
             P[i] = p_i
         return T, P, ss, F
 
@@ -484,13 +480,13 @@ class Spatial_Markov:
         distributions from the conditional and overall distributions.
         """
         n, t = self.y.shape
-        nt =  n * (t - 1)
+        nt = n * (t - 1)
         n0, n1, n2 = self.T.shape
         rn = range(n0)
-        mat = [self._ssmnp_test(self.s, self.S[i], self.T[i].sum()) for i in rn]
+        mat = [self._ssmnp_test(
+            self.s, self.S[i], self.T[i].sum()) for i in rn]
         return mat
 
-
     def _ssmnp_test(self, p1, p2, nt):
         """
         Steady state multinomial probability difference test
@@ -521,10 +517,10 @@ class Spatial_Markov:
         o = nt * p2
         e = nt * p1
         d = np.multiply((o - e), (o - e))
-        d = d/e
+        d = d / e
         chi2 = d.sum()
-        pvalue = 1-stats.chi2.cdf(chi2, k-1)
-        return (chi2,pvalue,k-1)
+        pvalue = 1 - stats.chi2.cdf(chi2, k - 1)
+        return (chi2, pvalue, k - 1)
 
     def _chi2_test(self):
         """
@@ -537,6 +533,7 @@ class Spatial_Markov:
         mat = [chi2(self.T[i], self.transitions) for i in rn]
         return mat
 
+
 def chi2(T1, T2):
     """
     chi-squared test of difference between two transition matrices.
@@ -582,16 +579,16 @@ def chi2(T1, T2):
            [   1.,    0.,   60.,  903.]])
     >>> chi2(T1,T2)
     (23.422628044813656, 0.0053137895983268457, 9)
-    
+
     Notes
     -----
 
     Second matrix is used to form the probabilities under the null.
     Marginal sums from first matrix are distributed across these probabilities
     under the null. In other words the observed transitions are taken from T1
-    while the expected transitions are formed as:
+    while the expected transitions are formed as follows
 
-        .. math::
+    .. math::
 
             E_{i,j} = \sum_j T1_{i,j} * T2_{i,j}/\sum_j T2_{i,j}
 
@@ -600,51 +597,48 @@ def chi2(T1, T2):
     """
     rs2 = T2.sum(axis=1)
     rs1 = T1.sum(axis=1)
-    rs2nz = rs2>0
-    rs1nz = rs1>0
+    rs2nz = rs2 > 0
+    rs1nz = rs1 > 0
     dof1 = sum(rs1nz)
     dof2 = sum(rs2nz)
     rs2 = rs2 + rs2nz
-    dof = (dof1-1) * (dof2-1)
-    p = np.diag(1/rs2) * np.matrix(T2)
+    dof = (dof1 - 1) * (dof2 - 1)
+    p = np.diag(1 / rs2) * np.matrix(T2)
     E = np.diag(rs1) * np.matrix(p)
     num = T1 - E
     num = np.multiply(num, num)
-    E = E + (E==0)
-    chi2 = num/E
+    E = E + (E == 0)
+    chi2 = num / E
     chi2 = chi2.sum()
-    pvalue = 1-stats.chi2.cdf(chi2, dof)
+    pvalue = 1 - stats.chi2.cdf(chi2, dof)
     return chi2, pvalue, dof
 
 
 class LISA_Markov(Markov):
     """
     Markov for Local Indicators of Spatial Association
-    
+
     Parameters
     ----------
 
     y  : array (n,t)
          n cross-sectional units observed over t time periods
-
     w  : weights instance
-
     permutations : int
                    number of permutations used to determine LISA significance
                    default = 0
-
     significance_level : float
                          significance level (two-sided) for filtering significant LISA end
                          points in a transition
                          default = 0.05
+
     Attributes
     ----------
     chi_2        : tuple (3 elements)
                    chi square test statistic
                    p-value
                    degrees of freedom
-                   for test that dynamics of y are independent of dynamics of
-                       wy
+                   for test that dynamics of y are independent of dynamics of wy
     classes      : array (4, 1)
                    1=HH, 2=LH, 3=LL, 4=HL (own, lag)
     expected_t   : array (4, 4)
@@ -655,6 +649,8 @@ class LISA_Markov(Markov):
                    occurred (q1 is quadrant in period 1, q2 is quadrant in
                    period 2)
 
+    .. Table:: Move Types
+
                    ==  ==     ========
                    q1  q2     move_type
                    ==  ==     ========
@@ -685,7 +681,9 @@ class LISA_Markov(Markov):
                        matrix (n, t-1)
                        integer values indicating the type and significance of a LISA
                        transition. st = 1 if significant in period t, else
-                       st=0 
+                       st=0
+
+    .. Table:: Significant Moves
 
                        ===============  ===================
                        (s1,s2)          move_type
@@ -723,13 +721,13 @@ class LISA_Markov(Markov):
                         4  3   1   0   31
                         4  4   1   0   32
                         1  1   0   1   33
-                        1  2   0   1   34 
+                        1  2   0   1   34
                         .  .   .   .    .
                         .  .   .   .    .
-                        4  3   0   1   47 
+                        4  3   0   1   47
                         4  4   0   1   48
-                        1  1   0   0   49 
-                        1  2   0   0   50 
+                        1  1   0   0   49
+                        1  2   0   0   50
                         .  .   .   .    .
                         .  .   .   .    .
                         4  3   0   0   63
@@ -740,14 +738,13 @@ class LISA_Markov(Markov):
                    ergodic distribution
     transitions  : matrix (4, 4)
                    count of transitions between each state i and j
-
     spillover    : binary array (n, 1)
                    locations that were not part of a cluster in period 1 but
                    joined a prexisting cluster in period 2
 
     Examples
     --------
- 
+
     >>> import numpy as np
     >>> f = pysal.open(pysal.examples.get_path("usjoin.csv"))
     >>> pci = np.array([f.by_col[str(y)] for y in range(1929,2010)]).transpose()
@@ -796,16 +793,18 @@ class LISA_Markov(Markov):
            [54, 54, 54, ..., 54, 54, 54],
            [49, 49, 49, ..., 54, 54, 54],
            [64, 64, 64, ..., 64, 64, 64]])
-        
+
     Any value less than 49 indicates at least one of the LISA end points was
     significant. So for example, the first spatial unit experienced a
-    transition of type 11 (LL, LL)  during the first three and last tree intervals (according to lm.move_types), however, the last three of these transitions involved insignificant LISAS in both the start and ending year of each transition.
-
+    transition of type 11 (LL, LL)  during the first three and last tree
+    intervals (according to lm.move_types), however, the last three of these
+    transitions involved insignificant LISAS in both the start and ending year
+    of each transition.
 
     Test whether the moves of y are independent of the moves of wy
 
     >>> lm.chi_2
-    (28645.587708342005, 0.0, 9)
+    (162.47505958346289, 0.0, 9)
 
     Actual transitions of LISAs
 
@@ -824,27 +823,27 @@ class LISA_Markov(Markov):
 
     >>> lm.expected_t
     array([[  1.12328098e+03,   1.15377356e+01,   3.47522158e-01,
-              3.41812865e+01],
+              3.38337644e+01],
            [  3.50272664e+00,   5.28473882e+02,   1.59178880e+01,
-              1.60233918e+01],
+              1.05503814e-01],
            [  1.53878082e-01,   2.32163556e+01,   1.46690710e+03,
-              1.47662977e+03],
-           [  9.67166213e+00,   9.74121200e+00,   6.15490790e+02,
-              1.22658713e+03]])
+              9.72266513e+00],
+           [  9.60775143e+00,   9.86856346e-02,   6.23537392e+00,
+              6.07058189e+02]])
     """
-    def __init__(self, y, w, permutations=0, 
-            significance_level=0.05):
+    def __init__(self, y, w, permutations=0,
+                 significance_level=0.05):
         y = y.transpose()
         pml = pysal.Moran_Local
 
         #################################################################
         # have to optimize conditional spatial permutations over a
         # time series - this is a place holder for the foreclosure paper
-        ml = [ pml(yi, w, permutations=permutations) for yi in y]
+        ml = [pml(yi, w, permutations=permutations) for yi in y]
         #################################################################
 
-        q = np.array([ mli.q for mli in ml]).transpose()
-        classes = np.arange(1,5) # no guarantee all 4 quadrants are visited
+        q = np.array([mli.q for mli in ml]).transpose()
+        classes = np.arange(1, 5)  # no guarantee all 4 quadrants are visited
         Markov.__init__(self, q, classes)
         self.q = q
         self.w = w
@@ -855,14 +854,14 @@ class LISA_Markov(Markov):
         sm = np.zeros((n, k), int)
         self.significance_level = significance_level
         if permutations > 0:
-            p = np.array([ mli.p_z_sim for mli in ml]).transpose()
+            p = np.array([mli.p_z_sim for mli in ml]).transpose()
             self.p_values = p
             pb = p <= significance_level
         else:
             pb = np.zeros_like(y.T)
         for t in range(k):
             origin = q[:, t]
-            dest = q[:, t+1]
+            dest = q[:, t + 1]
             p_origin = pb[:, t]
             p_dest = pb[:, t]
             for r in range(n):
@@ -870,23 +869,23 @@ class LISA_Markov(Markov):
                 key = (origin[r], dest[r], p_origin[r], p_dest[r])
                 sm[r, t] = MOVE_TYPES[key]
         if permutations > 0:
-            self.significant_moves = sm       
-        self.move_types=move_types
+            self.significant_moves = sm
+        self.move_types = move_types
 
         # null of own and lag moves being independent
 
         ybar = y.mean(axis=0)
-        r = y/ybar
-        ylag = np.array([ pysal.lag_spatial(w, yt) for yt in y])
-        rlag = ylag/ybar
+        r = y / ybar
+        ylag = np.array([pysal.lag_spatial(w, yt) for yt in y])
+        rlag = ylag / ybar
         rc = r < 1.
         rlagc = rlag < 1.
         markov_y = pysal.Markov(rc)
         markov_ylag = pysal.Markov(rlagc)
-        A = np.matrix( [[1, 0, 0, 0],
-            [0, 0, 1, 0],
-            [0, 0, 0, 1],
-            [0, 1, 0, 1]] )
+        A = np.matrix([[1, 0, 0, 0],
+                       [0, 0, 1, 0],
+                       [0, 0, 0, 1],
+                       [0, 1, 0, 0]])
 
         kp = A * np.kron(markov_y.p, markov_ylag.p) * A.T
         trans = self.transitions.sum(axis=1)
@@ -897,8 +896,7 @@ class LISA_Markov(Markov):
         self.expected_t = t1
         self.permutations = permutations
 
-
-    def spillover(self, quadrant = 1, neighbors_on=False):
+    def spillover(self, quadrant=1, neighbors_on=False):
         """
         Detect spillover locations for diffusion in LISA Markov
 
@@ -908,7 +906,7 @@ class LISA_Markov(Markov):
                    which quadrant in the scatterplot should form the core of a
                    cluster
 
-        neighbors_on: binary
+        neighbors_on : binary
                    If false then only the 1st order neighbors of a core
                    location are included in the cluster.
                    If true, neighbors of cluster core 1st order neighbors are
@@ -916,12 +914,11 @@ class LISA_Markov(Markov):
 
         Returns
         -------
-        dictionary: two keys: values pairs
-                    'components': array (n, t) 
+        dictionary : two keys - values pairs
+                    'components' - array (n, t)
                     values are integer ids (starting at 1) indicating which
                     component/cluster observation i in period t belonged to
-
-                    'spillover': array (n, t-1)
+                    'spillover' - array (n, t-1)
                     binary values indicating if the location was a spill-over
                     location that became a new member of a previously existing
                     cluster
@@ -950,7 +947,7 @@ class LISA_Markov(Markov):
                 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,
                 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,
                 0.,  0.,  1.,  0.,  0.,  1.,  0.,  0.,  0.])
-        
+
         Including neighbors of core neighbors
 
         >>> rn = lm_random.spillover(neighbors_on=True)
@@ -969,25 +966,26 @@ class LISA_Markov(Markov):
                 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  1.,  0.,  0.,
                 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,
                 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  1.])
-        
+
 
         """
         n, k = self.q.shape
         if self.permutations:
-            spill_over = np.zeros((n, k-1))
+            spill_over = np.zeros((n, k - 1))
             components = np.zeros((n, k))
-            i2id = {} # handle string keys
+            i2id = {}  # handle string keys
             for key in self.w.neighbors.keys():
-                id = self.w.id2i[key]
+                id = self.w.id2i[key]  # pylint "redefining built-in 'id'
                 i2id[id] = key
             sig_lisas = (self.q == quadrant) \
-                    * (self.p_values <= self.significance_level)
-            sig_ids = [ np.nonzero(sig_lisas[:,i])[0].tolist() for i in range(k)]
+                * (self.p_values <= self.significance_level)
+            sig_ids = [np.nonzero(
+                sig_lisas[:, i])[0].tolist() for i in range(k)]
 
             neighbors = self.w.neighbors
-            for t in range(k-1):
+            for t in range(k - 1):
                 s1 = sig_ids[t]
-                s2 = sig_ids[t+1]
+                s2 = sig_ids[t + 1]
                 g1 = pysal.region.components.Graph(undirected=True)
                 for i in s1:
                     for neighbor in neighbors[i2id[i]]:
@@ -996,7 +994,7 @@ class LISA_Markov(Markov):
                             for nn in neighbors[neighbor]:
                                 g1.add_edge(neighbor, nn, 1.0)
                 components1 = g1.connected_components(op=gt)
-                components1 = [ list(c.nodes) for c in components1 ]
+                components1 = [list(c.nodes) for c in components1]
                 g2 = pysal.region.components.Graph(undirected=True)
                 for i in s2:
                     for neighbor in neighbors[i2id[i]]:
@@ -1005,7 +1003,7 @@ class LISA_Markov(Markov):
                             for nn in neighbors[neighbor]:
                                 g2.add_edge(neighbor, nn, 1.0)
                 components2 = g2.connected_components(op=gt)
-                components2 = [ list(c.nodes) for c in components2 ]
+                components2 = [list(c.nodes) for c in components2]
                 c2 = []
                 c1 = []
                 for c in components2:
@@ -1013,11 +1011,11 @@ class LISA_Markov(Markov):
                 for c in components1:
                     c1.extend(c)
 
-                new_ids = [ j for j in c2 if j not in c1 ]
+                new_ids = [j for j in c2 if j not in c1]
                 spill_ids = []
                 for j in new_ids:
                     # find j's component in period 2
-                    cj = [ c for c in components2 if j in c][0]
+                    cj = [c for c in components2 if j in c][0]
                     # for members of j's component in period 2, check if they belonged to
                     # any components in period 1
                     for i in cj:
@@ -1030,9 +1028,7 @@ class LISA_Markov(Markov):
                 for c, component in enumerate(components1):
                     for i in component:
                         ii = self.w.id2i[i]
-                        components[ii, t] = c+1
-
-
+                        components[ii, t] = c + 1
             results = {}
             results['components'] = components
             results['spill_over'] = spill_over
@@ -1042,46 +1038,38 @@ class LISA_Markov(Markov):
             return None
 
 
-
-
 def kullback(F):
     """
     Kullback information based test of Markov Homogeneity
 
     Parameters
     ----------
-    F: array (s, r, r)
-       Values are transitions (not probabilities) for 
+    F : array (s, r, r)
+       Values are transitions (not probabilities) for
        s strata
        r initial states
-       r terminal states 
+       r terminal states
 
-    
     Returns
     -------
 
-    Results: Dictionary (key: value)
+    Results : Dictionary (key - value)
 
-        Conditional homogeneity: (float) test statistic for homogeneity of
+        Conditional homogeneity - (float) test statistic for homogeneity of
         transition probabilities across strata
 
-        Conditional homogeneity pvalue: (float) p-value for test statistic
-
-        Conditional homogeneity dof: (int) degrees of freedom =  r(s-1)(r-1)
+        Conditional homogeneity pvalue - (float) p-value for test statistic
 
+        Conditional homogeneity dof - (int) degrees of freedom =  r(s-1)(r-1)
 
     Notes
     -----
 
     Based on  Kullback, Kupperman and Ku (1962) [2]_
-    
+    Example below is taken from Table 9.2 
 
-    Example below is taken from Table 9.2 of [2]_
-
-
-
-    Example
-    -------
+    Examples
+    --------
 
     >>> s1 = np.array([
     ...         [ 22, 11, 24,  2,  2,  7],
@@ -1089,7 +1077,7 @@ def kullback(F):
     ...         [ 4, 21, 190, 25, 20, 34],
     ...         [0, 2, 14, 56, 14, 28],
     ...         [32, 15, 20, 10, 56, 14],
-    ...         [5, 22, 31, 18, 13, 134] 
+    ...         [5, 22, 31, 18, 13, 134]
     ...     ])
     >>> s2 = np.array([
     ...     [3, 6, 9, 3, 0, 8],
@@ -1099,7 +1087,7 @@ def kullback(F):
     ...     [22, 14, 9, 26, 224, 14],
     ...     [1, 5, 13, 53, 13, 116]
     ...     ])
-    >>> 
+    >>>
     >>> F = np.array([s1, s2])
     >>> res = kullback(F)
     >>> res['Conditional homogeneity']
@@ -1109,69 +1097,72 @@ def kullback(F):
     >>> res['Conditional homogeneity pvalue']
     0.0
 
-
     References
     ----------
 
     .. [2] Kullback, S. Kupperman, M. and H.H. Ku. (1962) "Tests for
-       contigency tables and Markov chains", Technometrics: 4, 573--608.
+       contigency tables and Markov chains", Technometrics : 4, 573--608.
 
     """
 
-    F1 = F==0
+    F1 = F == 0
     F1 = F + F1
     FLF = F * np.log(F1)
     T1 = 2 * FLF.sum()
 
     FdJK = F.sum(axis=0)
-    FdJK1 = FdJK + (FdJK==0)
+    FdJK1 = FdJK + (FdJK == 0)
     FdJKLFdJK = FdJK * np.log(FdJK1)
     T2 = 2 * FdJKLFdJK.sum()
 
     FdJd = F.sum(axis=0).sum(axis=1)
-    FdJd1 = FdJd + (FdJd==0)
-    T3 = 2 * (FdJd * np.log( FdJd1 )).sum()
+    FdJd1 = FdJd + (FdJd == 0)
+    T3 = 2 * (FdJd * np.log(FdJd1)).sum()
 
-    FIJd = F[:,:].sum(axis=1)
-    FIJd1 = FIJd + (FIJd==0)
+    FIJd = F[:, :].sum(axis=1)
+    FIJd1 = FIJd + (FIJd == 0)
     T4 = 2 * (FIJd * np.log(FIJd1)).sum()
 
     FIdd = F.sum(axis=1).sum(axis=1)
     T5 = 2 * (FIdd * np.log(FIdd)).sum()
 
-    T6 =  F.sum()
+    T6 = F.sum()
     T6 = 2 * T6 * np.log(T6)
 
-    s,r,r1 = F.shape
+    s, r, r1 = F.shape
     chom = T1 - T4 - T2 + T3
-    cdof = r * ( s - 1 ) * ( r - 1 )
+    cdof = r * (s - 1) * (r - 1)
     results = {}
     results['Conditional homogeneity'] = chom
     results['Conditional homogeneity dof'] = cdof
     results['Conditional homogeneity pvalue'] = 1 - stats.chi2.cdf(chom, cdof)
     return results
 
-def prais(pmat):		
-    """		
+
+def prais(pmat):
+    """
     Prais conditional mobility measure
 
     Parameters
     ----------
 
-    pmat: kxk matrix
+    pmat : kxk matrix
           Markov probability transition matrix
 
     Returns
     -------
-    
+
     pr : 1xk matrix
-          Conditional mobility measures for each of the k classes with each
-          element  obtained as:
+          Conditional mobility measures for each of the k classes.
 
-          .. math::
+    Notes
+    -----
 
-                pr_i = 1 - \sum_j p_{i,j}
+    Prais' conditional mobility measure for a class is defined as:
 
+    .. math::
+
+            pr_i = 1 - \sum_j p_{i,j}
 
     Examples
     --------
@@ -1196,34 +1187,38 @@ def prais(pmat):
     >>> pysal.spatial_dynamics.markov.prais(m.p)
     matrix([[ 0.08988764,  0.21468144,  0.21125   ,  0.20194986,  0.07259074]])
 
-               
+    """
+    pr = (pmat.sum(axis=1) - np.diag(pmat))[0]
+    return pr
 
 
-    """		
-    pr = (pmat.sum(axis=1)-np.diag(pmat))[0]
-    return 	pr	
-		
-def shorrock(pmat):		
-    """		
-    Shorrocks mobility measure		
+def shorrock(pmat):
+    """
+    Shorrocks mobility measure
 
     Parameters
     ----------
 
-    pmat: kxk matrix
+    pmat : kxk matrix
           Markov probability transition matrix
 
     Returns
     -------
-    
-    sh : scalar
-          Conditional mobility measure:
 
-        .. math::
+    sh : scalar
+          Conditional mobility measure
+
+
+
+    Notes
+    -----
+
+    Shorock's mobility measure is defined as
+
+    .. math::
 
          sh = (k  - \sum_{j=1}^{k} p_{j,j})/(k - 1)
 
-
     Examples
     --------
     >>> import numpy as np
@@ -1247,15 +1242,10 @@ def shorrock(pmat):
     >>> pysal.spatial_dynamics.markov.shorrock(m.p)
     0.19758992000997844
 
-    """		
-    t=np.trace(pmat)		
-    k=pmat.shape[1]		
-    sh = (k-t) / (k-1)
+    """
+    t = np.trace(pmat)
+    k = pmat.shape[1]
+    sh = (k - t) / (k - 1)
     return sh
-		
-def test():
-    import doctest
-    doctest.testmod(verbose=True)
 
-if __name__ == '__main__':
-    test()
+
