@@ -2,6 +2,9 @@ import unittest
 import pysal
 from pysal.spatial_dynamics import interaction
 import numpy as np
+import scipy
+
+scp_version = int(scipy.version.version.split(".")[1])
 
 
 class SpaceTimeEvents_Tester(unittest.TestCase):
@@ -22,7 +25,8 @@ class Knox_Tester(unittest.TestCase):
 
     def test_knox(self):
         result = interaction.knox(
-            self.events, delta=20, tau=5, permutations=1)
+            self.events.space,
+            self.events.t, delta=20, tau=5, permutations=1)
         self.assertEquals(result['stat'], 13.0)
 
 
@@ -32,8 +36,8 @@ class Mantel_Tester(unittest.TestCase):
         self.events = interaction.SpaceTimeEvents(path, 'T')
 
     def test_mantel(self):
-        result = interaction.mantel(self.events, 1, scon=0.0,
-                                    spow=1.0, tcon=0.0, tpow=1.0)
+        result = interaction.mantel(self.events.space,
+                self.events.time, 1, scon=0.0, spow=1.0, tcon=0.0, tpow=1.0)
         self.assertAlmostEquals(result['stat'], 0.014154, 6)
 
 
@@ -43,8 +47,13 @@ class Jacquez_Tester(unittest.TestCase):
         self.events = interaction.SpaceTimeEvents(path, 'T')
 
     def test_jacquez(self):
-        result = interaction.jacquez(self.events, k=3, permutations=1)
-        self.assertEquals(result['stat'], 13)
+        result = interaction.jacquez(self.events.space,
+                self.events.t, k=3, permutations=1)
+        if scp_version > 11:
+            self.assertEquals(result['stat'], 12)
+        else:
+            self.assertEquals(result['stat'], 13)
+            
 
 
 class ModifiedKnox_Tester(unittest.TestCase):
@@ -54,7 +63,8 @@ class ModifiedKnox_Tester(unittest.TestCase):
 
     def test_modified_knox(self):
         result = interaction.modified_knox(
-            self.events, delta=20, tau=5, permutations=1)
+            self.events.space,
+            self.events.t, delta=20, tau=5, permutations=1)
         self.assertAlmostEquals(result['stat'], 2.810160, 6)
 
 
