@@ -9,7 +9,9 @@ from utils import spdot, sphstack, RegressionPropsY, RegressionPropsVM
 __author__ = "Luc Anselin luc.anselin@asu.edu, David C. Folch david.folch@asu.edu, Jing Yao jingyao@asu.edu"
 __all__ = ["TSLS"]
 
+
 class BaseTSLS(RegressionPropsY, RegressionPropsVM):
+
     """
     Two stage least squares (2SLS) (note: no consistency checks,
     diagnostics or constant added)
@@ -101,7 +103,7 @@ class BaseTSLS(RegressionPropsY, RegressionPropsVM):
     pfora1a2     : array
                    n(zthhthi)'varb
 
-    
+
     Examples
     --------
 
@@ -126,55 +128,60 @@ class BaseTSLS(RegressionPropsY, RegressionPropsVM):
      [  0.5200379 ]
      [ -1.58216593]]
     >>> reg = BaseTSLS(y, X, yd, q=q, robust="white")
-    
+
     """
-    def __init__(self, y, x, yend, q=None, h=None,\
+
+    def __init__(self, y, x, yend, q=None, h=None,
                  robust=None, gwk=None, sig2n_k=False):
 
-        if issubclass(type(q), np.ndarray) and issubclass(type(h), np.ndarray):  
+        if issubclass(type(q), np.ndarray) and issubclass(type(h), np.ndarray):
             raise Exception, "Please do not provide 'q' and 'h' together"
-        if q==None and h==None:
+        if q == None and h == None:
             raise Exception, "Please provide either 'q' or 'h'"
-        
-        self.y = y  
+
+        self.y = y
         self.n = y.shape[0]
         self.x = x
 
         self.kstar = yend.shape[1]
-        z = sphstack(self.x,yend)  # including exogenous and endogenous variables   
+        # including exogenous and endogenous variables
+        z = sphstack(self.x, yend)
         if type(h).__name__ not in ['ndarray', 'csr_matrix']:
-            h = sphstack(self.x,q)   # including exogenous variables and instrument
+            # including exogenous variables and instrument
+            h = sphstack(self.x, q)
         self.z = z
         self.h = h
         self.q = q
         self.yend = yend
-        self.k = z.shape[1]    # k = number of exogenous variables and endogenous variables
-        hth = spdot(h.T,h)    
+        # k = number of exogenous variables and endogenous variables
+        self.k = z.shape[1]
+        hth = spdot(h.T, h)
         hthi = la.inv(hth)
-        zth = spdot(z.T,h)    
-        hty = spdot(h.T,y) 
-        
-        factor_1 = np.dot(zth,hthi)  
-        factor_2 = np.dot(factor_1,zth.T)  
-        varb = la.inv(factor_2)          # this one needs to be in cache to be used in AK
-        factor_3 = np.dot(varb,factor_1)   
-        betas = np.dot(factor_3,hty)  
+        zth = spdot(z.T, h)
+        hty = spdot(h.T, y)
+
+        factor_1 = np.dot(zth, hthi)
+        factor_2 = np.dot(factor_1, zth.T)
+        # this one needs to be in cache to be used in AK
+        varb = la.inv(factor_2)
+        factor_3 = np.dot(varb, factor_1)
+        betas = np.dot(factor_3, hty)
         self.betas = betas
         self.varb = varb
-        self.zthhthi = factor_1  
-        
+        self.zthhthi = factor_1
+
         # predicted values
-        self.predy = spdot(z,betas)
-        
+        self.predy = spdot(z, betas)
+
         # residuals
         u = y - self.predy
         self.u = u
-        
-        # attributes used in property 
+
+        # attributes used in property
         self.hth = hth     # Required for condition index
         self.hthi = hthi   # Used in error models
         self.htz = zth.T
-      
+
         if robust:
             self.vm = ROBUST.robust_vm(reg=self, gwk=gwk, sig2n_k=sig2n_k)
 
@@ -187,16 +194,19 @@ class BaseTSLS(RegressionPropsY, RegressionPropsVM):
     @property
     def pfora1a2(self):
         if 'pfora1a2' not in self._cache:
-            self._cache['pfora1a2'] = self.n*np.dot(self.zthhthi.T, self.varb) 
-        return self._cache['pfora1a2']    
-            
+            self._cache['pfora1a2'] = self.n * \
+                np.dot(self.zthhthi.T, self.varb)
+        return self._cache['pfora1a2']
+
     @property
     def vm(self):
         if 'vm' not in self._cache:
             self._cache['vm'] = np.dot(self.sig2, self.varb)
         return self._cache['vm']
 
+
 class TSLS(BaseTSLS):
+
     """
     Two stage least squares with results and diagnostics.
 
@@ -339,7 +349,7 @@ class TSLS(BaseTSLS):
     pfora1a2     : array
                    n(zthhthi)'varb
 
-    
+
     Examples
     --------
 
@@ -357,7 +367,7 @@ class TSLS(BaseTSLS):
     data in using any method.  
 
     >>> db = pysal.open(pysal.examples.get_path("columbus.dbf"),'r')
-    
+
     Extract the CRIME column (crime rates) from the DBF file and make it the
     dependent variable for the regression. Note that PySAL requires this to be
     an numpy array of shape (n, 1) as opposed to the also common shape of (n, )
@@ -405,12 +415,13 @@ class TSLS(BaseTSLS):
      [ -1.58216593]]
 
     """
-    def __init__(self, y, x, yend, q,\
-                 w=None,\
-                 robust=None, gwk=None, sig2n_k=False,\
-                 spat_diag=False,\
-                 vm=False, name_y=None, name_x=None,\
-                 name_yend=None, name_q=None,\
+
+    def __init__(self, y, x, yend, q,
+                 w=None,
+                 robust=None, gwk=None, sig2n_k=False,
+                 spat_diag=False,
+                 vm=False, name_y=None, name_x=None,
+                 name_yend=None, name_q=None,
                  name_w=None, name_gwk=None, name_ds=None):
 
         n = USER.check_arrays(y, x, yend, q)
@@ -419,9 +430,9 @@ class TSLS(BaseTSLS):
         USER.check_robust(robust, gwk)
         USER.check_spat_diag(spat_diag, w)
         x_constant = USER.check_constant(x)
-        BaseTSLS.__init__(self, y=y, x=x_constant, yend=yend, q=q,\
-                              robust=robust, gwk=gwk, sig2n_k=sig2n_k)
-        self.title = "TWO STAGE LEAST SQUARES"        
+        BaseTSLS.__init__(self, y=y, x=x_constant, yend=yend, q=q,
+                          robust=robust, gwk=gwk, sig2n_k=sig2n_k)
+        self.title = "TWO STAGE LEAST SQUARES"
         self.name_ds = USER.set_name_ds(name_ds)
         self.name_y = USER.set_name_y(name_y)
         self.name_x = USER.set_name_x(name_x, x)
@@ -433,12 +444,12 @@ class TSLS(BaseTSLS):
         self.name_w = USER.set_name_w(name_w, w)
         self.name_gwk = USER.set_name_w(name_gwk, gwk)
         SUMMARY.TSLS(reg=self, vm=vm, w=w, spat_diag=spat_diag)
-        
+
 
 def _test():
     import doctest
     start_suppress = np.get_printoptions()['suppress']
-    np.set_printoptions(suppress=True)    
+    np.set_printoptions(suppress=True)
     doctest.testmod()
     np.set_printoptions(suppress=start_suppress)
 
@@ -448,9 +459,9 @@ if __name__ == '__main__':
 
     import numpy as np
     import pysal
-    db = pysal.open(pysal.examples.get_path("columbus.dbf"),'r')
+    db = pysal.open(pysal.examples.get_path("columbus.dbf"), 'r')
     y_var = 'CRIME'
-    y = np.array([db.by_col(y_var)]).reshape(49,1)
+    y = np.array([db.by_col(y_var)]).reshape(49, 1)
     x_var = ['INC']
     x = np.array([db.by_col(name) for name in x_var]).T
     yd_var = ['HOVAL']
@@ -459,6 +470,6 @@ if __name__ == '__main__':
     q = np.array([db.by_col(name) for name in q_var]).T
     w = pysal.rook_from_shapefile(pysal.examples.get_path("columbus.shp"))
     w.transform = 'r'
-    tsls = TSLS(y, x, yd, q, w=w, spat_diag=True, name_y=y_var, name_x=x_var, name_yend=yd_var, name_q=q_var, name_ds='columbus', name_w='columbus.gal')
+    tsls = TSLS(y, x, yd, q, w=w, spat_diag=True, name_y=y_var, name_x=x_var,
+                name_yend=yd_var, name_q=q_var, name_ds='columbus', name_w='columbus.gal')
     print tsls.summary
-
