@@ -27,14 +27,15 @@ def set_name_ds(name_ds):
 
     Returns
     -------
-    
+
     name_ds     : string
-                  
+
     """
     if not name_ds:
         name_ds = 'unknown'
     return name_ds
 
+
 def set_name_y(name_y):
     """Set the dataset name in regression; return generic name if user
     provides no explicit name."
@@ -47,14 +48,15 @@ def set_name_y(name_y):
 
     Returns
     -------
-    
+
     name_ds     : string
-                  
+
     """
     if not name_y:
         name_y = 'dep_var'
     return name_y
 
+
 def set_name_x(name_x, x, constant=False):
     """Set the independent variable names in regression; return generic name if user
     provides no explicit name."
@@ -73,18 +75,19 @@ def set_name_x(name_x, x, constant=False):
 
     Returns
     -------
-    
+
     name_x      : list of strings
-                  
+
     """
     if not name_x:
-        name_x = ['var_'+str(i+1) for i in range(x.shape[1])]
+        name_x = ['var_' + str(i + 1) for i in range(x.shape[1])]
     else:
         name_x = name_x[:]
     if not constant:
         name_x.insert(0, 'CONSTANT')
     return name_x
-    
+
+
 def set_name_yend(name_yend, yend):
     """Set the endogenous variable names in regression; return generic name if user
     provides no explicit name."
@@ -97,18 +100,19 @@ def set_name_yend(name_yend, yend):
 
     Returns
     -------
-    
+
     name_yend   : list of strings
-                  
+
     """
     if yend != None:
         if not name_yend:
-            return ['endogenous_'+str(i+1) for i in range(len(yend[0]))]
+            return ['endogenous_' + str(i + 1) for i in range(len(yend[0]))]
         else:
             return name_yend[:]
     else:
         return []
 
+
 def set_name_q(name_q, q):
     """Set the external instrument names in regression; return generic name if user
     provides no explicit name."
@@ -123,18 +127,19 @@ def set_name_q(name_q, q):
 
     Returns
     -------
-    
+
     name_q      : list of strings
-                  
+
     """
     if q != None:
         if not name_q:
-            return ['instrument_'+str(i+1) for i in range(len(q[0]))]
+            return ['instrument_' + str(i + 1) for i in range(len(q[0]))]
         else:
             return name_q[:]
     else:
         return []
 
+
 def set_name_yend_sp(name_y):
     """Set the spatial lag name in regression; return generic name if user
     provides no explicit name."
@@ -147,12 +152,13 @@ def set_name_yend_sp(name_y):
 
     Returns
     -------
-    
+
     name_yend_sp : string
-                  
+
     """
     return 'W_' + name_y
 
+
 def set_name_q_sp(name_x, w_lags, name_q, lag_q, force_all=False):
     """Set the spatial instrument names in regression; return generic name if user
     provides no explicit name."
@@ -168,25 +174,26 @@ def set_name_q_sp(name_x, w_lags, name_q, lag_q, force_all=False):
 
     Returns
     -------
-    
+
     name_q_sp   : list of strings
-                  
+
     """
     if force_all:
         names = name_x
     else:
-        names = name_x[1:] # drop the constant
+        names = name_x[1:]  # drop the constant
     if lag_q:
-        names = names + name_q   
+        names = names + name_q
     sp_inst_names = []
     for j in names:
-        sp_inst_names.append('W_'+j)
+        sp_inst_names.append('W_' + j)
     if w_lags > 1:
-        for i in range(2, w_lags+1):
+        for i in range(2, w_lags + 1):
             for j in names:
-                sp_inst_names.append('W'+str(i)+'_'+j)
+                sp_inst_names.append('W' + str(i) + '_' + j)
     return sp_inst_names
-    
+
+
 def set_name_h(name_x, name_q):
     """Set the full instruments names in regression; return generic name if user
     provides no explicit name."
@@ -201,12 +208,13 @@ def set_name_h(name_x, name_q):
 
     Returns
     -------
-    
+
     name_h      : list of strings
-                  
+
     """
     return name_x + name_q
 
+
 def set_robust(robust):
     """Return generic name if user passes None to the robust parameter in a
     regression. Note: already verified that the name is valid in
@@ -220,14 +228,15 @@ def set_robust(robust):
 
     Returns
     -------
-    
+
     robust      : string
-                  
+
     """
     if not robust:
         return 'unadjusted'
     return robust
 
+
 def set_name_w(name_w, w):
     """Return generic name if user passes None to the robust parameter in a
     regression. Note: already verified that the name is valid in
@@ -243,9 +252,9 @@ def set_name_w(name_w, w):
 
     Returns
     -------
-    
+
     name_w      : string
-                  
+
     """
     if w != None:
         if name_w != None:
@@ -255,7 +264,7 @@ def set_name_w(name_w, w):
     return None
 
 
-def set_name_multi(multireg,multi_set,name_multiID,y,x,name_y,name_x,name_ds,title,name_w,robust,endog=False,sp_lag=False):
+def set_name_multi(multireg, multi_set, name_multiID, y, x, name_y, name_x, name_ds, title, name_w, robust, endog=False, sp_lag=False):
     """Returns multiple regression objects with generic names
 
     Parameters
@@ -275,25 +284,28 @@ def set_name_multi(multireg,multi_set,name_multiID,y,x,name_y,name_x,name_ds,tit
     name_multiID = set_name_ds(name_multiID)
     if endog or sp_lag:
         name_yend = set_name_yend(endog[2], endog[0])
-        name_q = set_name_q(endog[3], endog[1])        
+        name_q = set_name_q(endog[3], endog[1])
     for r in multi_set:
-        multireg[r].title = title + "%s" %r
+        multireg[r].title = title + "%s" % r
         multireg[r].name_ds = name_ds
         multireg[r].robust = set_robust(robust)
-        multireg[r].name_w = name_w        
-        multireg[r].name_y = '%s_%s'%(str(r), name_y)
-        multireg[r].name_x = ['%s_%s'%(str(r), i) for i in name_x]
+        multireg[r].name_w = name_w
+        multireg[r].name_y = '%s_%s' % (str(r), name_y)
+        multireg[r].name_x = ['%s_%s' % (str(r), i) for i in name_x]
         multireg[r].name_multiID = name_multiID
         if endog or sp_lag:
-            multireg[r].name_yend = ['%s_%s'%(str(r), i) for i in name_yend]
-            multireg[r].name_q = ['%s_%s'%(str(r), i) for i in name_q]
+            multireg[r].name_yend = ['%s_%s' % (str(r), i) for i in name_yend]
+            multireg[r].name_q = ['%s_%s' % (str(r), i) for i in name_q]
             if sp_lag:
-                multireg[r].name_yend.append(set_name_yend_sp(multireg[r].name_y))
-                multireg[r].name_q.extend(set_name_q_sp(multireg[r].name_x, sp_lag[0], multireg[r].name_q, sp_lag[1]))
+                multireg[r].name_yend.append(
+                    set_name_yend_sp(multireg[r].name_y))
+                multireg[r].name_q.extend(
+                    set_name_q_sp(multireg[r].name_x, sp_lag[0], multireg[r].name_q, sp_lag[1]))
             multireg[r].name_z = multireg[r].name_x + multireg[r].name_yend
             multireg[r].name_h = multireg[r].name_x + multireg[r].name_q
     return multireg
 
+
 def check_arrays(*arrays):
     """Check if the objects passed by a user to a regression class are
     correctly structured. If the user's data is correctly formed this function
@@ -306,7 +318,7 @@ def check_arrays(*arrays):
     *arrays : anything
               Objects passed by the user to a regression class; any type
               object can be passed and any number of objects can be passed
-     
+
     Returns
     -------
 
@@ -350,6 +362,7 @@ def check_arrays(*arrays):
         raise Exception, "arrays not all of same length"
     return rows[0]
 
+
 def check_y(y, n):
     """Check if the y object passed by a user to a regression class is
     correctly structured. If the user's data is correctly formed this function
@@ -365,7 +378,7 @@ def check_y(y, n):
 
     n       : int
               number of observations
-     
+
     Returns
     -------
 
@@ -396,6 +409,7 @@ def check_y(y, n):
     if shape != (n, 1):
         raise Exception, "y must be a single column array matching the length of other arrays"
 
+
 def check_weights(w, y, w_required=False):
     """Check if the w parameter passed by the user is a pysal.W object and
     check that its dimensionality matches the y parameter.  Note that this
@@ -416,7 +430,7 @@ def check_weights(w, y, w_required=False):
 
     Returns : nothing
               Nothing is returned
-              
+
     Examples
     --------
 
@@ -437,7 +451,7 @@ def check_weights(w, y, w_required=False):
     """
     if w_required == True or w != None:
         if w == None:
-            raise Exception, "A weights matrix w must be provided to run this method."            
+            raise Exception, "A weights matrix w must be provided to run this method."
         if not isinstance(w, pysal.W):
             raise Exception, "w must be a pysal.W object"
         if w.n != y.shape[0]:
@@ -469,7 +483,7 @@ def check_robust(robust, wk):
 
     Returns : nothing
               Nothing is returned
-              
+
     Examples
     --------
 
@@ -509,14 +523,15 @@ def check_robust(robust, wk):
                 if vmin < 0.0:
                     raise Exception, "Off-diagonal entries must be greater than or equal to 0."
                 if vmax > 1.0:
-                    ##### NOTE: we are not checking for the case of exactly 1.0 #####
+                    # NOTE: we are not checking for the case of exactly 1.0 ###
                     raise Exception, "Off-diagonal entries must be less than 1."
-        elif robust.lower() == 'white':
+        elif robust.lower() == 'white' or robust.lower() == 'ogmm':
             if wk:
                 raise Exception, "White requires that wk be set to None"
         else:
             raise Exception, "invalid value passed to robust, see docs for valid options"
 
+
 def check_spat_diag(spat_diag, w):
     """Check if there is a w parameter passed by the user if the user also
     requests spatial diagnostics.
@@ -535,7 +550,7 @@ def check_spat_diag(spat_diag, w):
 
     Returns : nothing
               Nothing is returned
-              
+
     Examples
     --------
 
@@ -558,7 +573,8 @@ def check_spat_diag(spat_diag, w):
         if type(w).__name__ != 'W':
             raise Exception, "w must be a pysal.W object to run spatial diagnostics"
 
-def check_regimes(reg_set):
+
+def check_regimes(reg_set, N=None, K=None):
     """Check if there are at least two regimes
 
     Parameters
@@ -572,10 +588,13 @@ def check_regimes(reg_set):
 
     Returns : nothing
               Nothing is returned
-              
+
     """
     if len(reg_set) < 2:
         raise Exception, "At least 2 regimes are needed to run regimes methods. Please check your regimes variable."
+    if 1.0 * N / len(reg_set) < K + 1:
+        raise Exception, "There aren't enough observations for the given number of regimes and variables. Please check your regimes variable."
+
 
 def check_constant(x):
     """Check if the X matrix contains a constant, raise exception if it does
@@ -592,7 +611,7 @@ def check_constant(x):
 
     Returns : nothing
               Nothing is returned
-              
+
     Examples
     --------
 
@@ -612,7 +631,7 @@ def check_constant(x):
         raise Exception, "x array cannot contain a constant vector; constant will be added automatically"
     else:
         x_constant = COPY.copy(x)
-        return sphstack(np.ones((x_constant.shape[0],1)),x_constant)
+        return sphstack(np.ones((x_constant.shape[0], 1)), x_constant)
 
 
 def _test():
@@ -621,6 +640,3 @@ def _test():
 
 if __name__ == '__main__':
     _test()
-
-
-
