@@ -6,15 +6,15 @@ __author__ = "Sergio J. Rey <srey@asu.edu> "
 
 import pysal
 import scipy.spatial
-from pysal.common import *
+from pysal.common import KDTree
 from pysal.weights import W
-
-from scipy import sparse
+import scipy.stats
+import numpy as np
 
 __all__ = ["knnW", "Kernel", "DistanceBand"]
 
 
-def knnW(data, k=2, p=2, ids=None):
+def knnW(data, k=2, p=2, ids=None, pct_unique=0.25):
     """
     Creates nearest neighbor weights matrix based on k nearest
     neighbors.
@@ -34,13 +34,16 @@ def knnW(data, k=2, p=2, ids=None):
                  1: Manhattan distance
     ids        : list
                  identifiers to attach to each observation
+    pct_unique : float
+                 threshold percentage of unique points in data. Below this
+                 threshold tree is built on unique values only
+
     Returns
     -------
 
     w         : W instance
                 Weights object with binary weights
 
-
     Examples
     --------
 
@@ -50,25 +53,46 @@ def knnW(data, k=2, p=2, ids=None):
     >>> data=np.hstack([x,y])
     >>> wnn2=knnW(data,k=2)
     >>> wnn4=knnW(data,k=4)
-    >>> wnn4.neighbors[0]
-    [1, 5, 6, 2]
-    >>> wnn4.neighbors[5]
-    [0, 6, 10, 1]
-    >>> wnn2.neighbors[0]
-    [1, 5]
-    >>> wnn2.neighbors[5]
-    [0, 6]
-    >>> wnn2.pct_nonzero
-    0.080000000000000002
+    >>> set([1,5,6,2]) == set(wnn4.neighbors[0])
+    True
+    >>> set([0,6,10,1]) == set(wnn4.neighbors[5])
+    True
+    >>> set([1,5]) == set(wnn2.neighbors[0])
+    True
+    >>> set([0,6]) == set(wnn2.neighbors[5])
+    True
+    >>> "%.2f"%wnn2.pct_nonzero
+    '0.08'
     >>> wnn4.pct_nonzero
     0.16
     >>> wnn3e=knnW(data,p=2,k=3)
-    >>> wnn3e.neighbors[0]
-    [1, 5, 6]
+    >>> set([1,5,6]) == set(wnn3e.neighbors[0])
+    True
     >>> wnn3m=knnW(data,p=1,k=3)
-    >>> wnn3m.neighbors[0]
-    [1, 5, 2]
+    >>> a = set([1,5,2])
+    >>> b = set([1,5,6])
+    >>> c = set([1,5,10])
+    >>> w0n = set(wnn3m.neighbors[0])
+    >>> a==w0n or b==w0n or c==w0n
+    True
 
+    ids
+
+    >>> wnn2 = knnW(data,2)
+    >>> wnn2[0]
+    {1: 1.0, 5: 1.0}
+    >>> wnn2[1]
+    {0: 1.0, 2: 1.0}
+
+    now with 1 rather than 0 offset
+
+    >>> wnn2 = knnW(data,2, ids = range(1,26))
+    >>> wnn2[1]
+    {2: 1.0, 6: 1.0}
+    >>> wnn2[2]
+    {1: 1.0, 3: 1.0}
+    >>> 0 in wnn2.neighbors
+    False
 
     Notes
     -----
@@ -78,38 +102,53 @@ def knnW(data, k=2, p=2, ids=None):
     See Also
     --------
     pysal.weights.W
+
     """
-    # handle data
+
     if issubclass(type(data), scipy.spatial.KDTree):
         kd = data
         data = kd.data
+        nnq = kd.query(data, k=k+1, p=p)
+        info = nnq[1]
     elif type(data).__name__ == 'ndarray':
-        kd = KDTree(data)
+        # check if unique points are a small fraction of all points
+        ind =  np.lexsort(data.T)
+        u = data[np.concatenate(([True],np.any(data[ind[1:]]!=data[ind[:-1]],axis=1)))]
+        pct_u = len(u)*1. / len(data)
+        if pct_u < pct_unique:
+            tree = KDTree(u)
+            nnq = tree.query(data, k=k+1, p=p)
+            info = nnq[1]
+            uid = [np.where((data == ui).all(axis=1))[0][0] for ui in u]
+            new_info = np.zeros((len(data), k + 1), 'int')
+            for i, row in enumerate(info):
+                new_info[i] = [uid[j] for j in row]
+            info = new_info
+        else:
+            kd = KDTree(data)
+            # calculate
+            nnq = kd.query(data, k=k + 1, p=p)
+            info = nnq[1]
     else:
-        print 'Unsupported  type'
+        print 'Unsupported type'
+        return None
 
-    # calculate
-    nnq = kd.query(data, k=k + 1, p=p)
-    info = nnq[1]
     neighbors = {}
-    weights = {}
-    if ids:
-        idset = np.array(ids)
-    else:
-        idset = np.arange(len(info))
     for i, row in enumerate(info):
         row = row.tolist()
-        row.remove(i)
-        neighbors[idset[i]] = list(idset[row])
-        weights[idset[i]] = [1] * len(neighbors[idset[i]])
-
-    return pysal.weights.W(neighbors, weights=weights, id_order=ids)
+        if i in row:
+            row.remove(i)
+            focal = i
+        if ids:
+            row = [ ids[j] for j in row]
+            focal = ids[i]
+        neighbors[focal] = row
+    return pysal.weights.W(neighbors,  id_order=ids)
 
 
 class Kernel(W):
     """Spatial weights based on kernel functions
 
-
     Parameters
     ----------
 
@@ -127,6 +166,10 @@ class Kernel(W):
                   where :math:`dknn` is a vector of k-nearest neighbor
                   distances (the distance to the kth nearest neighbor for each
                   observation).  For adaptive bandwidths, :math:`h_i=dknn_i`
+    diagonal    : boolean
+                  If true, set diagonal weights = 1.0, if false (default),
+                  diagonals weights are set to value according to kernel
+                  function.
     function    : string {'triangular','uniform','quadratic','quartic','gaussian'}
                   kernel function defined as follows with
 
@@ -152,14 +195,12 @@ class Kernel(W):
 
                       K(z) = (3/4)(1-z^2) \ if |z| \le 1
 
-
                   quartic
 
                   .. math::
 
                       K(z) = (15/16)(1-z^2)^2 \ if |z| \le 1
 
-
                   gaussian
 
                   .. math::
@@ -242,9 +283,19 @@ class Kernel(W):
            [ 11.18034101],
            [ 14.14213704],
            [ 18.02775818]])
+
+    Diagonals to 1.0
+
+    >>> kq = Kernel(points,function='gaussian')
+    >>> kq.weights
+    {0: [0.3989422804014327, 0.35206533556593145, 0.3412334260702758], 1: [0.35206533556593145, 0.3989422804014327, 0.2419707487162134, 0.3412334260702758, 0.31069657591175387], 2: [0.2419707487162134, 0.3989422804014327, 0.31069657591175387], 3: [0.3412334260702758, 0.3412334260702758, 0.3989422804014327, 0.3011374490937829, 0.26575287272131043], 4: [0.31069657591175387, 0.31069657591175387, 0.3011374490937829, 0.3989422804014327, 0.35206533556593145], 5: [0.26575287272131043, 0.35206533556593145, 0.3989422804014327]}
+    >>> kqd = Kernel(points, function='gaussian', diagonal=True)
+    >>> kqd.weights
+    {0: [1.0, 0.35206533556593145, 0.3412334260702758], 1: [0.35206533556593145, 1.0, 0.2419707487162134, 0.3412334260702758, 0.31069657591175387], 2: [0.2419707487162134, 1.0, 0.31069657591175387], 3: [0.3412334260702758, 0.3412334260702758, 1.0, 0.3011374490937829, 0.26575287272131043], 4: [0.31069657591175387, 0.31069657591175387, 0.3011374490937829, 1.0, 0.35206533556593145], 5: [0.26575287272131043, 0.35206533556593145, 1.0]}
     """
     def __init__(self, data, bandwidth=None, fixed=True, k=2,
-                 function='triangular', eps=1.0000001, ids=None):
+                 function='triangular', eps=1.0000001, ids=None,
+                 diagonal=False):
         if issubclass(type(data), scipy.spatial.KDTree):
             self.kdt = data
             self.data = self.kdt.data
@@ -268,6 +319,9 @@ class Kernel(W):
 
         self._eval_kernel()
         neighbors, weights = self._k_to_W(ids)
+        if diagonal:
+            for i in neighbors:
+                weights[i][neighbors[i].index(i)] = 1.0
         W.__init__(self, neighbors, weights, ids)
 
     def _k_to_W(self, ids=None):
@@ -309,7 +363,6 @@ class Kernel(W):
                          bwi in enumerate(self.bandwidth)]
             self.neigh = neighbors
         # get distances for neighbors
-        data = np.array(self.data)
         bw = self.bandwidth
 
         kdtq = self.kdt.query
@@ -321,17 +374,17 @@ class Kernel(W):
         zs = z
         # functions follow Anselin and Rey (2010) table 5.4
         if self.function == 'triangular':
-            self.kernel = [1 - z for z in zs]
+            self.kernel = [1 - zi for zi in zs]  
         elif self.function == 'uniform':
-            self.kernel = [np.ones(z.shape) * 0.5 for z in zs]
+            self.kernel = [np.ones(zi.shape) * 0.5 for zi in zs]
         elif self.function == 'quadratic':
-            self.kernel = [(3. / 4) * (1 - z ** 2) for z in zs]
+            self.kernel = [(3. / 4) * (1 - zi ** 2) for zi in zs]
         elif self.function == 'quartic':
-            self.kernel = [(15. / 16) * (1 - z ** 2) ** 2 for z in zs]
+            self.kernel = [(15. / 16) * (1 - zi ** 2) ** 2 for zi in zs]
         elif self.function == 'gaussian':
             c = np.pi * 2
             c = c ** (-0.5)
-            self.kernel = [c * np.exp(-(z ** 2) / 2.) for z in zs]
+            self.kernel = [c * np.exp(-(zi ** 2) / 2.) for zi in zs]
         else:
             print 'Unsupported kernel function', self.function
 
@@ -365,14 +418,23 @@ class DistanceBand(W):
 
     >>> points=[(10, 10), (20, 10), (40, 10), (15, 20), (30, 20), (30, 30)]
     >>> w=DistanceBand(points,threshold=11.2)
+    WARNING: there is one disconnected observation (no neighbors)
+    Island id:  [2]
     >>> w.weights
     {0: [1, 1], 1: [1, 1], 2: [], 3: [1, 1], 4: [1], 5: [1]}
     >>> w.neighbors
-    {0: [1, 3], 1: [0, 3], 2: [], 3: [0, 1], 4: [5], 5: [4]}
+    {0: [1, 3], 1: [0, 3], 2: [], 3: [1, 0], 4: [5], 5: [4]}
+    >>> w=DistanceBand(points,threshold=14.2)
+    >>> w.weights
+    {0: [1, 1], 1: [1, 1, 1], 2: [1], 3: [1, 1], 4: [1, 1, 1], 5: [1]}
+    >>> w.neighbors
+    {0: [1, 3], 1: [0, 3, 4], 2: [4], 3: [1, 0], 4: [5, 1, 2], 5: [4]}
 
     inverse distance weights
 
     >>> w=DistanceBand(points,threshold=11.2,binary=False)
+    WARNING: there is one disconnected observation (no neighbors)
+    Island id:  [2]
     >>> w.weights[0]
     [0.10000000000000001, 0.089442719099991588]
     >>> w.neighbors[0]
@@ -382,10 +444,11 @@ class DistanceBand(W):
     gravity weights
 
     >>> w=DistanceBand(points,threshold=11.2,binary=False,alpha=-2.)
+    WARNING: there is one disconnected observation (no neighbors)
+    Island id:  [2]
     >>> w.weights[0]
     [0.01, 0.0079999999999999984]
 
-
     Notes
     -----
 
@@ -423,63 +486,47 @@ class DistanceBand(W):
         """
         find all pairs within threshold
         """
-        kd = self.kd
-        #ns=[kd.query_ball_point(point,self.threshold) for point in self.data]
-        ns = kd.query_ball_tree(kd, self.threshold)
-        self._nmat = ns
+        self.dmat = self.kd.sparse_distance_matrix(
+                self.kd, max_distance=self.threshold)
 
     def _distance_to_W(self, ids=None):
-        allneighbors = {}
-        weights = {}
         if ids:
             ids = np.array(ids)
         else:
-            ids = np.arange(len(self._nmat))
+            ids = np.arange(self.dmat.shape[0])
+        neighbors = dict([(i,[]) for i in ids])
+        weights = dict([(i,[]) for i in ids])
         if self.binary:
-            for i, neighbors in enumerate(self._nmat):
-                ns = [ni for ni in neighbors if ni != i]
-                neigh = list(ids[ns])
-                if len(neigh) == 0:
-                    allneighbors[ids[i]] = []
-                    weights[ids[i]] = []
-                else:
-                    allneighbors[ids[i]] = neigh
-                    weights[ids[i]] = [1] * len(ns)
+            for key,weight in self.dmat.items():
+                i,j = key
+                if j not in neighbors[i]:
+                    weights[i].append(1)
+                    neighbors[i].append(j)
+                if i not in neighbors[j]:
+                    weights[j].append(1)
+                    neighbors[j].append(i)
+
         else:
-            self.dmat = self.kd.sparse_distance_matrix(
-                self.kd, max_distance=self.threshold)
-            for i, neighbors in enumerate(self._nmat):
-                ns = [ni for ni in neighbors if ni != i]
-                neigh = list(ids[ns])
-                if len(neigh) == 0:
-                    allneighbors[ids[i]] = []
-                    weights[ids[i]] = []
-                else:
-                    allneighbors[ids[i]] = neigh
-                    weights[ids[i]] = [self.dmat[(
-                        i, j)] ** self.alpha for j in ns]
-        return allneighbors, weights
+            for key,weight in self.dmat.items():
+                i,j = key
+                if j not in neighbors[i]:
+                    weights[i].append(weight**self.alpha)
+                    neighbors[i].append(j)
+                if i not in neighbors[j]:
+                    weights[j].append(weight**self.alpha)
+                    neighbors[j].append(i)
+
+        return neighbors, weights
 
 
+def _test():
+    import doctest
+    # the following line could be used to define an alternative to the '<BLANKLINE>' flag
+    #doctest.BLANKLINE_MARKER = 'something better than <BLANKLINE>'
+    start_suppress = np.get_printoptions()['suppress']
+    np.set_printoptions(suppress=True)
+    doctest.testmod()
+    np.set_printoptions(suppress=start_suppress)
 
-if __name__ == "__main__":
-
-    x = np.arange(1, 5)
-
-    n = len(x)
-    data = []
-    row = []
-    col = []
-    rdata = []
-
-    for i in range(n - 1):
-        data.extend(x[i + 1:])
-
-        rdata.extend((x[i] * np.ones((1, n - i - 1))).tolist()[0]
-                     )
-        row.extend([i] * (n - i - 1))
-        col.extend(range(i + 1, n))
-    L = sparse.csr_matrix((data, (row, col)), shape=(n, n))
-    R = sparse.csr_matrix((rdata, (row, col)), shape=(n, n))
-    dlr = L - R
-    D = dlr.multiply(dlr)
+if __name__ == '__main__':
+    _test()
